@@ -1,4 +1,4 @@
-/*$Id: fkHttpChannel.cpp,v 1.17 2007/07/22 21:03:11 jwrobel Exp $*/
+/*$Id: fkHttpChannel.cpp,v 1.20 2007/08/17 20:47:05 jwrobel Exp $*/
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
  *
@@ -33,87 +33,89 @@
 
 /*
  * Create new fkHttpChannel
- * @origChannel 
+ * @origChannel
  *             Mozilla's Channel that will be doing almost all work here.
- * @sentence 
+ * @sentence
  *             Used to pass information about whitelisted channel that
  *             shouldn't be checked.
- * @connRules 
+ * @connRules
  *             Rules set for this channel
  */
-fkHttpChannel::fkHttpChannel(nsIChannel *origChannel, fkISentence *sentence, 
+fkHttpChannel::fkHttpChannel(nsIChannel *origChannel, fkISentence *sentence,
 			     ActiveRuleSet *connRules)
 {
 	TRACE("constructor %08x %08x", this, origChannel);
 	nsresult rv;
-	
+
 	this->origChannel = origChannel;
 	this->sentence = sentence;
 	this->connRules = connRules;
-	
-	origRequest = do_QueryInterface(origChannel, &rv); 
+
+	requestHashTable->Put(origChannel, this);
+
+	origRequest = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsIRequest");
 
-	origHttpChannel = do_QueryInterface(origChannel, &rv); 
+	origHttpChannel = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsIHttpChannel");
 
-	origRequestObserver = do_QueryInterface(origChannel, &rv); 
+	origRequestObserver = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsIRequestObserver");
 
-	origStreamListener = do_QueryInterface(origChannel, &rv); 
+	origStreamListener = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsIStreamListener");
 
-	origCachingChannel = do_QueryInterface(origChannel, &rv); 
+	origCachingChannel = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsICachingChannel");
 
-	origUploadChannel = do_QueryInterface(origChannel, &rv); 
+	origUploadChannel = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsIUploadChannel");
 
-	origCacheListener = do_QueryInterface(origChannel, &rv); 
+	origCacheListener = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsICacheListener");
 
-	origEncodedChannel = do_QueryInterface(origChannel, &rv); 
+	origEncodedChannel = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsIEncodedChannel");
 
-	origHttpChannelInternal = do_QueryInterface(origChannel, &rv); 
+	origHttpChannelInternal = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsIHttpChannelInternal");
 
-	origTransportEventSink = do_QueryInterface(origChannel, &rv); 
+	origTransportEventSink = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsITransportEventSink");
 
-	origResumableChannel = do_QueryInterface(origChannel, &rv); 
+	origResumableChannel = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsIResumableChannel");
 
-	origSupportsPriority = do_QueryInterface(origChannel, &rv); 
+	origSupportsPriority = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsISupportsPriority");
 
-	origProtocolProxyCallback = do_QueryInterface(origChannel, &rv); 
+	origProtocolProxyCallback = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsIProtocolProxyCallback");
 
-	origPropertyBag = do_QueryInterface(origChannel, &rv); 
+	origPropertyBag = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsIPropertyBag");
 
-	origPropertyBag2 = do_QueryInterface(origChannel, &rv); 
+	origPropertyBag2 = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsIPropertyBag2");
 
-	origWritablePropertyBag = do_QueryInterface(origChannel, &rv); 
+	origWritablePropertyBag = do_QueryInterface(origChannel, &rv);
 	checkResult(rv, "nsIWritablePropertyBag");
 
-	origWritablePropertyBag2 = do_QueryInterface(origChannel, &rv); 
-	checkResult(rv, "nsIWritablePropertyBag2");	
+	origWritablePropertyBag2 = do_QueryInterface(origChannel, &rv);
+	checkResult(rv, "nsIWritablePropertyBag2");
 
 	callbacksProxy = nsnull;
 	loadGroupProxy = nsnull;
-	
+
 #ifdef DEBUG
 	nsCOMPtr<nsILoadGroup> lg;
 	rv = origRequest->GetLoadGroup(getter_AddRefs(lg));
 	FK_ASSERT(!lg);
 
-	nsCOMPtr<nsIInterfaceRequestor> cb;	
+	nsCOMPtr<nsIInterfaceRequestor> cb;
 	rv = origChannel->GetNotificationCallbacks(getter_AddRefs(cb));
 	FK_ASSERT(!cb);
-#endif	
+#endif
 
 }
 
@@ -122,15 +124,15 @@ fkHttpChannel::~fkHttpChannel()
 	TRACE("destructor %08x", this);
 
 #ifdef DEBUG
-	FK_ASSERT(requestHashTable->Get(origHttpChannel, nsnull));
-#endif	
-	requestHashTable->Remove(origHttpChannel);
+	FK_ASSERT(requestHashTable->Get(origChannel, nsnull));
+#endif
+	requestHashTable->Remove(origChannel);
 	TRACE("HashTable count %d", requestHashTable->Count());
 	if (this->connRules)
 		delete connRules;
 }
 
-NS_IMETHODIMP 
+NS_IMETHODIMP
 fkHttpChannel::init()
 {
 	return NS_OK;
@@ -143,7 +145,7 @@ NS_IMPL_RELEASE(fkHttpChannel)
 /*Only interfaces implemented by origLoadGroup are returned.*/
 NS_IMETHODIMP
 fkHttpChannel::QueryInterface(const nsIID &aIID, void **aResult)
-{	
+{
 	if (aResult == NULL) {
 		return NS_ERROR_NULL_POINTER;
 	}
@@ -190,7 +192,7 @@ fkHttpChannel::QueryInterface(const nsIID &aIID, void **aResult)
 		foundInterface = NS_STATIC_CAST(nsIWritablePropertyBag2*, this);
 	}
 
-	
+
 	if (!foundInterface) {
 		TRACE("fkHttpChannel: interface not implemented %s .", aIID.ToString());
 #ifdef DEBUG
@@ -200,10 +202,10 @@ fkHttpChannel::QueryInterface(const nsIID &aIID, void **aResult)
 
 			/*This leaks memory but only in debug mode when FK_ASSERT will be
 			  triggered anyway*/
-			nsresult rv = origChannel->QueryInterface(aIID, &dummy);			
+			nsresult rv = origChannel->QueryInterface(aIID, &dummy);
 			if (!NS_FAILED(rv) && dummy){
-				TRACE("origChannel implements this!!!!");				
-				FK_ASSERT(dummy);
+				TRACE("origChannel implements this!!!!");
+				FK_ASSERT(!dummy);
 			}
 		}
 #endif
@@ -214,7 +216,7 @@ fkHttpChannel::QueryInterface(const nsIID &aIID, void **aResult)
 	NS_ADDREF(foundInterface);
 	*aResult = foundInterface;
 	return NS_OK;
-} 
+}
 
 #else
 NS_INTERFACE_MAP_BEGIN(fkHttpChannel)
@@ -232,7 +234,7 @@ NS_INTERFACE_MAP_BEGIN(fkHttpChannel)
     NS_INTERFACE_MAP_ENTRY(nsITransportEventSink)
     NS_INTERFACE_MAP_ENTRY(nsISupportsPriority)
     NS_INTERFACE_MAP_ENTRY(nsIProtocolProxyCallback)
-	
+
     NS_INTERFACE_MAP_ENTRY(nsIWritablePropertyBag)
     NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsIPropertyBag, nsIWritablePropertyBag)
     NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIWritablePropertyBag)
@@ -261,7 +263,7 @@ fkHttpChannel::IsPending(PRBool *value)
 
 NS_IMETHODIMP
 fkHttpChannel::GetStatus(nsresult *aStatus)
-{	
+{
 	TRACE("");
 	return origRequest->GetStatus(aStatus);
 }
@@ -293,7 +295,7 @@ fkHttpChannel::Resume()
 
 NS_IMETHODIMP
 fkHttpChannel::GetLoadGroup(nsILoadGroup **aLoadGroup)
-{	
+{
 	TRACE("fkHttpChannel");
 	NS_ENSURE_ARG_POINTER(aLoadGroup);
 	if (!loadGroupProxy){
@@ -311,32 +313,33 @@ NS_IMETHODIMP
 fkHttpChannel::SetLoadGroup(nsILoadGroup *aLoadGroup)
 {
 	nsresult rv;
-	TRACE("setting load group %08x origChannel = %08x this = %08x", 
-	      aLoadGroup, &*origChannel, this);	
+	TRACE("setting load group %08x origChannel = %08x this = %08x",
+	      aLoadGroup, &*origChannel, this);
 	loadGroupProxy = nsnull;
-	
+
 	if (aLoadGroup){
 		/*If this cast succeeds aLoadGroup is already a proxy (instance of fkLoadGroup).
-		  We should get and use original LoadGroup from this proxy to avoid 
-		  call chains like this:		  
+		  Original LoadGroup from this proxy should be used to avoid
+		  call chains like this:
 		  proxy1->OnStartRequest(nsRequest...) calls proxy2->OnStartRequest(fkReqeuest1)
-		  calls origObserver->OnStartReqeust(fkRequest2) (This is wrong, fkReqeust1 has 
+		  calls origObserver->OnStartReqeust(fkRequest2) (This is wrong, fkReqeust1 has
 		  started not fkRequest2).
 		*/
 		nsCOMPtr<fkILoadGroupProxy> tmp = do_QueryInterface(aLoadGroup, &rv);
 		if (!NS_FAILED(rv)){
 			TRACE("proxy already used");
 			loadGroupProxy = aLoadGroup;
-		}
-		else{
+		}else{
 			fkLoadGroup *proxy;
 			if (!groupHashTable->Get(aLoadGroup, &proxy)){
 				TRACE("group not found");
-				proxy = new fkLoadGroup(aLoadGroup);		
-				if (!proxy || NS_FAILED(proxy->init()))
+				proxy = new fkLoadGroup(aLoadGroup);
+				if (!proxy)
 					return NS_ERROR_OUT_OF_MEMORY;
-				groupHashTable->Put(aLoadGroup, proxy);
-
+				if (NS_FAILED((rv = proxy->init()))){
+					delete proxy;
+					return rv;
+				}
 			}
 			else{
 				TRACE("group found");
@@ -351,25 +354,15 @@ fkHttpChannel::SetLoadGroup(nsILoadGroup *aLoadGroup)
 NS_IMETHODIMP
 fkHttpChannel::GetLoadFlags(nsLoadFlags *aLoadFlags)
 {
-	nsresult rv = origRequest->GetLoadFlags(aLoadFlags);
-	TRACE("flags = %08x", *aLoadFlags);
-	if (*aLoadFlags & nsIChannel::LOAD_REPLACE){
-		TRACE("fkHttpChannel REPLACE FLAG SET");
-	}
-	FK_ASSERT(!NS_FAILED(rv));
-	return rv;
+	TRACE("");
+	return origRequest->GetLoadFlags(aLoadFlags);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::SetLoadFlags(nsLoadFlags aLoadFlags)
 {
-	nsresult rv = origRequest->SetLoadFlags(aLoadFlags);
-	TRACE("fkHttpChannel flags = %08x", aLoadFlags);
-	if (aLoadFlags & nsIChannel::LOAD_REPLACE){
-		TRACE("fkHttpChannel REPLACE FLAG SET");
-	}
-	FK_ASSERT(!NS_FAILED(rv));
-	return rv;
+	TRACE("");
+	return origRequest->SetLoadFlags(aLoadFlags);
 }
 
 //-----------------------------------------------------------------------------
@@ -413,15 +406,14 @@ fkHttpChannel::SetOwner(nsISupports *owner)
 
 NS_IMETHODIMP
 fkHttpChannel::GetNotificationCallbacks(nsIInterfaceRequestor **callbacks)
-{	
-	
+{
 	TRACE("fkHttpChannel");
 	NS_ENSURE_ARG_POINTER(callbacks);
 	if (!callbacksProxy){
 		*callbacks = nsnull;
 		return NS_OK;
 	}
-	
+
 	nsresult rv;
 	nsCOMPtr<fkICallbacksProxy> p = do_QueryInterface(callbacksProxy, &rv);
 	FK_ASSERT(!NS_FAILED(rv));
@@ -432,11 +424,11 @@ NS_IMETHODIMP
 fkHttpChannel::SetNotificationCallbacks(nsIInterfaceRequestor *callbacks)
 {
 	nsresult rv;
-	TRACE("Setting callbacks %08x origChannel = %08x this = %08x", 
-	      callbacks, &*origChannel, this);	
+	TRACE("Setting callbacks %08x origChannel = %08x this = %08x",
+	      callbacks, &*origChannel, this);
 	callbacksProxy = nsnull;
-	
-	if (callbacks){	
+
+	if (callbacks){
 		nsCOMPtr<fkICallbacksProxy> tmp = do_QueryInterface(callbacks, &rv);
 		if (!NS_FAILED(rv)){
 			TRACE("proxy already used");
@@ -445,10 +437,9 @@ fkHttpChannel::SetNotificationCallbacks(nsIInterfaceRequestor *callbacks)
 			fkCallbacks *proxy;
 			if (!callbacksHashTable->Get(callbacks, &proxy)){
 				TRACE("callbacks not found");
-				proxy = new fkCallbacks(nsnull, callbacks);		
+				proxy = new fkCallbacks(nsnull, callbacks);
 				if (!proxy)
 					return NS_ERROR_OUT_OF_MEMORY;
-				callbacksHashTable->Put(callbacks, proxy);
 			}
 			else{
 				TRACE("callbacks found");
@@ -458,7 +449,6 @@ fkHttpChannel::SetNotificationCallbacks(nsIInterfaceRequestor *callbacks)
 	}
 	return origChannel->SetNotificationCallbacks(callbacksProxy);
 }
-	
 
 
 NS_IMETHODIMP
@@ -473,14 +463,14 @@ fkHttpChannel::GetContentType(nsACString &value)
 {
 	TRACE("");
 	return origChannel->GetContentType(value);
-#if 0	/* 
+#if 0	/*
 	   Remains of attempts to replace blocked traffic.
 	   Maybe someday in a future ...
 	 */
 	int action;
 	sentence->GetAction(&action);
 	TRACE("action = %d", action);
-	
+
 	if (0 || action == sentence->CANCEL){
 		value = nsEmbedCString("text/html");
 		return NS_OK;
@@ -533,8 +523,10 @@ fkHttpChannel::SetContentLength(PRInt32 value)
 NS_IMETHODIMP
 fkHttpChannel::Open(nsIInputStream **_retval)
 {
-	/*TODO: Browser doesn't use this, but some extensions may*/
-	TRACE("Not tracing Open calls");
+	/* TODO: trace Open() calls. They are used for example
+	   when [Tools]->[Addons] is clicked */
+	TRACE("TODO: Not tracing Open calls");
+	//FK_ASSERT(false);
 	return origChannel->Open(_retval);
 }
 
@@ -544,17 +536,17 @@ fkHttpChannel::Open(nsIInputStream **_retval)
  */
 NS_IMETHODIMP
 fkHttpChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *context)
-{	
+{
 	int action;
 	nsresult rv;
-	
+
 	TRACE("");
 	NS_ENSURE_ARG_POINTER(listener);
-	
+
 	sentence->GetAction(&action);
 	if (action == sentence->BLOCK)
 		return NS_ERROR_ABORT;
-	if (action == sentence->DONT_AUDIT){		
+	if (action == sentence->DONT_AUDIT){
 		/*TODO: can't happen*/
 		TRACE("whitelisted");
 		FK_ASSERT(false);
@@ -568,21 +560,20 @@ fkHttpChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *context)
 		   old channel is used in AsyncOpen Call */
 		return origChannel->AsyncOpen(listener, context);
 	}
-	
 	TRACE("checking");
-	
+
 	fkStreamListener *fklistener_raw =
 		new fkStreamListener(listener, this, sentence, connRules);
-	
+
 	if (!fklistener_raw)
 		return NS_ERROR_OUT_OF_MEMORY;
 
 	nsCOMPtr<nsIStreamListener> fklistener;
-	rv = fklistener_raw->QueryInterface(NS_GET_IID(nsIStreamListener), 
+	rv = fklistener_raw->QueryInterface(NS_GET_IID(nsIStreamListener),
 					    getter_AddRefs(fklistener));
 	if (NS_FAILED(rv))
 		return rv;
-	
+
 	/*Call Mozilla AsyncOpen implementation but with Firekeeper
 	  StreamListener*/
 	return origChannel->AsyncOpen(fklistener, context);
@@ -766,7 +757,7 @@ fkHttpChannel::GetUploadStream(nsIInputStream **stream)
 
 
 NS_IMETHODIMP
-fkHttpChannel::SetUploadStream(nsIInputStream *stream, 
+fkHttpChannel::SetUploadStream(nsIInputStream *stream,
 			       const nsACString &contentType, PRInt32 contentLength)
 {
 	TRACE("");
@@ -863,14 +854,16 @@ NS_IMETHODIMP
 fkHttpChannel::OnStartRequest(nsIRequest *request, nsISupports *ctxt)
 {
 	TRACE("");
-	return origRequestObserver->OnStartRequest(this, ctxt);
+	FK_ASSERT(false);
+	return origRequestObserver->OnStartRequest(tryGetRequestProxy(request), ctxt);
 }
 
 NS_IMETHODIMP
 fkHttpChannel::OnStopRequest(nsIRequest *request, nsISupports *ctxt, nsresult status)
 {
 	TRACE("");
-	return origRequestObserver->OnStopRequest(this, ctxt, status);
+	FK_ASSERT(false);
+	return origRequestObserver->OnStopRequest(tryGetRequestProxy(request), ctxt, status);
 }
 
 //-----------------------------------------------------------------------------
@@ -883,7 +876,8 @@ fkHttpChannel::OnDataAvailable(nsIRequest *request, nsISupports *ctxt,
                                PRUint32 offset, PRUint32 count)
 {
 	TRACE("");
-	return origStreamListener->OnDataAvailable(this, ctxt, input,
+	FK_ASSERT(false);
+	return origStreamListener->OnDataAvailable(tryGetRequestProxy(request), ctxt, input,
 						   offset, count);
 }
 
@@ -898,7 +892,7 @@ fkHttpChannel::OnTransportStatus(nsITransport *trans, nsresult status,
 	TRACE("");
 	return origTransportEventSink->OnTransportStatus(trans, status, progress,
 							 progressMax);
-} 
+}
 
 //-----------------------------------------------------------------------------
 // fkHttpChannel::nsICachingChannel
