@@ -1,4 +1,4 @@
-/*$Id: fkStreamListener.cpp,v 1.22 2007/07/22 20:05:21 jwrobel Exp $*/
+/*$Id: fkStreamListener.cpp,v 1.24 2007/08/16 12:06:19 jwrobel Exp $*/
 
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
@@ -44,11 +44,11 @@
   accurate. But because fragments have a random size, it is impossible for
   an attacker to guess where is a border of a fragment and to format a
   response in such a way that suspicious data is always spread between two
-  fragments and undetected. 
+  fragments and undetected.
   Following improvement further increases accuracy: part
   of a data from the end of a fragment (also of a random size) is
-  appended to the beginning of a next fragment. 
-  It is possible to bypass this system only for attacks which detection 
+  appended to the beginning of a next fragment.
+  It is possible to bypass this system only for attacks which detection
   requires processing of a large amount of data. But essentially
   attacks have short fingerprints and can be easily detected
   with this approach.
@@ -61,12 +61,12 @@
 /*Minimum number of bytes that are appended from the end of a previous
   fragment to the beginning of a next one*/
 #define NO_FLUSH_POINT_MIN 100
-#define NO_FLUSH_POINT_MAX 200  
+#define NO_FLUSH_POINT_MAX 200
 
 #define REQUEST request
 
 NS_IMPL_ISUPPORTS4(fkStreamListener, nsIStreamListener,
-		   nsIRequestObserver, 
+		   nsIRequestObserver,
 		   nsIHttpHeaderVisitor,
 		   fkIStreamListenerProxy);
 
@@ -74,36 +74,35 @@ NS_IMPL_ISUPPORTS4(fkStreamListener, nsIStreamListener,
   fkStreamListener constructor
   @origListener
           Original listener to which not dangerous data is passed
-  @channel	     
+  @channel
           fkHttpChannel that created this StreamListener
   @connRules
-          ActiveRuleSet used to check data received via thia stream          
+          ActiveRuleSet used to check data received via thia stream
  */
 fkStreamListener::fkStreamListener(nsIStreamListener *origListener, 
 				   fkHttpChannel *channel,
-				   fkISentence *sentence, 
+				   fkISentence *sentence,
 				   ActiveRuleSet *connRules):
-	body(nsnull), headers(nsnull), url(nsnull), body_len(0), 
-	headers_len(0), body_max(BODY_INIT_SIZE), 
+	body(nsnull), headers(nsnull), url(nsnull), body_len(0),
+	headers_len(0), body_max(BODY_INIT_SIZE),
 	headers_max(HEADERS_INIT_SIZE), offset(0), debugNoHeaders(true),
 	flush_point(0), no_flush_point(0)
-{	
-	TRACE("constructor %08x origListener = %08x", 
+{
+	TRACE("constructor %08x origListener = %08x",
 	      this, origListener);
-	
+
 	this->origListener = origListener;
 	this->sentence = sentence;
 	this->connRules = connRules;
 	this->channel = channel;
 	NS_ADDREF(this->channel);
 
-	
 	body = (char *)nsMemory::Alloc(body_max);
 	if (body == nsnull){
 		body_max = headers_max = 0;
 		return;
 	}
-	
+
 	headers = (char *)nsMemory::Alloc(headers_max);
 	if (headers == nsnull){
 		body_max = headers_max = 0;
@@ -120,7 +119,7 @@ fkStreamListener::~fkStreamListener()
 {
 	TRACE("destructor %08x", this);
 	NS_RELEASE(this->channel);
-	
+
 	if (body != nsnull)
 		nsMemory::Free(body);
 	if (headers != nsnull)
@@ -133,7 +132,7 @@ fkStreamListener::~fkStreamListener()
 void fkStreamListener::drawFlushPoint()
 {
 	srand(flush_point + time(0) + (int)this);
-	flush_point = FLUSH_POINT_MIN + 
+	flush_point = FLUSH_POINT_MIN +
 		rand() % (FLUSH_POINT_MAX - FLUSH_POINT_MIN);
 	TRACE("new flush point %d", flush_point);
 }
@@ -144,16 +143,16 @@ void fkStreamListener::drawFlushPoint()
 void fkStreamListener::drawNoFlushPoint()
 {
 	srand(no_flush_point + time(0) + (int)this);
-	no_flush_point = NO_FLUSH_POINT_MIN + 
+	no_flush_point = NO_FLUSH_POINT_MIN +
 		rand() % (NO_FLUSH_POINT_MAX - NO_FLUSH_POINT_MIN);
-	
+
 	TRACE("new no flush point %d", no_flush_point);
 }
 
 /*
   Get URL from which this listener receives data.
  */
-const char* 
+const char*
 fkStreamListener::getURL()
 {
 	if (!url.Length()){
@@ -162,7 +161,6 @@ fkStreamListener::getURL()
 		uri->GetSpec(url);
 	}
 	return url.get();
-	
 }
 
 /*
@@ -173,7 +171,7 @@ fkStreamListener::headers_append(const char *src, PRUint32 len)
 {
 	if (headers == nsnull)
 		return NS_ERROR_OUT_OF_MEMORY;
-	
+
 	if (expandBuf(&headers, len, headers_len, headers_max) != NS_OK)
 		return NS_ERROR_OUT_OF_MEMORY;
 
@@ -184,7 +182,7 @@ fkStreamListener::headers_append(const char *src, PRUint32 len)
 
 /*
   Expand buffer if needed.
-  @buf 
+  @buf
       buffer
   @len
       number of needed bytes
@@ -196,8 +194,8 @@ fkStreamListener::headers_append(const char *src, PRUint32 len)
 NS_IMETHODIMP
 fkStreamListener::expandBuf(char **buf, PRUint32 len, PRUint32 end, PRUint32 &max_size)
 {
-	while(max_size  - end < len){	
-		max_size *= 2;		
+	while(max_size  - end < len){
+		max_size *= 2;
 		char *tmp = (char *)nsMemory::Realloc(*buf, max_size);
 		if (tmp == nsnull){
 			max_size /= 2;
@@ -205,6 +203,8 @@ fkStreamListener::expandBuf(char **buf, PRUint32 len, PRUint32 end, PRUint32 &ma
 		}
 		*buf = tmp;
 	}
+	TRACE("%08x buffer expanded to %d body_max = %d",
+	      this, max_size, body_max);
 	return NS_OK;
 }
 
@@ -219,9 +219,9 @@ fkStreamListener::checkBody()
 		if (!rule_raw){
 			return NS_ERROR_OUT_OF_MEMORY;
 		}
-		nsCOMPtr<fkIRule> 
+		nsCOMPtr<fkIRule>
 			rule(do_QueryInterface(rule_raw));
-		fkJudge->Judge(getURL(), rule, sentence);		
+		fkJudge->Judge(getURL(), rule, sentence);
 		int action;
 		sentence->GetAction(&action);
 		TRACE("action = %d", action);
@@ -243,7 +243,7 @@ fkStreamListener::checkBody()
 /*
   Check body fragment and pass it to the original listener if it is ok.
   @buffer
-         body fragment to check 
+         body fragment to check
  */
 NS_IMETHODIMP
 fkStreamListener::do_OnDataAvailable(nsIRequest *request, nsISupports *aContext)
@@ -256,43 +256,42 @@ fkStreamListener::do_OnDataAvailable(nsIRequest *request, nsISupports *aContext)
 		return rv;
 	}
 	TRACE("check body ok");
-	
+
 	nsCOMPtr<nsIByteArrayInputStream> convertedStreamSup;
-	
-	int data_len = body_len - no_flush_point; 
+
+	int data_len = body_len - no_flush_point;
 	char *lBuf = (char *) nsMemory::Alloc(data_len);
 	if (!lBuf)
 		return NS_ERROR_OUT_OF_MEMORY;
-	memcpy(lBuf, body, data_len);	
-	
-	rv = NS_NewByteArrayInputStream(getter_AddRefs(convertedStreamSup), 
+	memcpy(lBuf, body, data_len);
+
+	rv = NS_NewByteArrayInputStream(getter_AddRefs(convertedStreamSup),
 					lBuf, data_len);
 	if (NS_FAILED(rv))
 		return rv;
-	
-	nsCOMPtr<nsIInputStream> convertedStream = 
+
+	nsCOMPtr<nsIInputStream> convertedStream =
 		do_QueryInterface(convertedStreamSup, &rv);
 	if (NS_FAILED(rv))
 		return rv;
-	
+
 	if (no_flush_point){
 		/*Copy no_flush_point bytes from the end of previous body fragment
 		  to the beginning of a new one*/
-		memmove(body, body + data_len, no_flush_point); 
+		memmove(body, body + data_len, no_flush_point);
 		body_len = no_flush_point;
-	}
-	else{
+	}else{
 		body_len = 0;
 	}
-	
-	rv = origListener->OnDataAvailable(getRequestProxy(request), aContext, 
-					   convertedStream, 
+
+	rv = origListener->OnDataAvailable(tryGetRequestProxy(request), aContext,
+					   convertedStream,
 					   offset, data_len);
 	offset += data_len;;
 	return rv;
 }
 
-/*  
+/*
     Chack last fragment of a response body and pass it to the oryginal listener
     if it is ok. Free body buffer.
  */
@@ -302,42 +301,47 @@ fkStreamListener::do_LastOnDataAvailable(nsIRequest *request, nsISupports* acont
 	nsresult rv;
 	FK_ASSERT(body != nsnull);
 	TRACE("calling checkBody");
-	
+
 	rv = checkBody();
 	if (NS_FAILED(rv))
 		return rv;
 
 	nsCOMPtr<nsIByteArrayInputStream> convertedStreamSup;
-	
+
 	/*this stream becomes owner of a body buffer*/
-	rv = NS_NewByteArrayInputStream(getter_AddRefs(convertedStreamSup), 
-					body, body_len);    
+	rv = NS_NewByteArrayInputStream(getter_AddRefs(convertedStreamSup),
+					body, body_max);
 	if (NS_FAILED(rv))
 		return rv;
-	
+
+	TRACE("%08x", body);
+
 	body = nsnull;
-	
-	nsCOMPtr<nsIInputStream> convertedStream = 
+	nsCOMPtr<nsIInputStream> convertedStream =
 		do_QueryInterface(convertedStreamSup, &rv);
 	if (NS_FAILED(rv))
 		return rv;
-	
+
 	TRACE("OK offset = %d len %d", offset, body_len);
-	return origListener->OnDataAvailable(getRequestProxy(request), acontext, 
-					     convertedStream, offset, 
-					     body_len);
+	TRACE("%08x %08x %08x %08x", &*convertedStreamSup, &*convertedStream, acontext, request);
+
+	rv = origListener->OnDataAvailable(tryGetRequestProxy(request), acontext,
+					   convertedStream, offset,
+					   body_len);
+	body_len = 0;
+	return rv;
 }
 
 //-----------------------------------------------------------------------------
 // fkStreamListener::fkIStreamListenerProxy
 //-----------------------------------------------------------------------------
-NS_IMETHODIMP 
+NS_IMETHODIMP
 fkStreamListener::GetOrigStreamListener(nsIStreamListener **aStreamListener)
 {
 	TRACE("fkStreamListener::fkIStreamListenerProxy");
-	NS_ENSURE_ARG_POINTER(aStreamListener);		
+	NS_ENSURE_ARG_POINTER(aStreamListener);
 	FK_ASSERT(origListener);
-	
+
 	NS_IF_ADDREF(*aStreamListener = origListener);
 	return NS_OK;
 }
@@ -349,49 +353,49 @@ NS_IMETHODIMP
 fkStreamListener::OnDataAvailable(nsIRequest *request, nsISupports *ctxt,
 				  nsIInputStream *input,
 				  PRUint32 off, PRUint32 count)
-{	
-	TRACE("got data count = %d offset = %d request = %08x origchannel = %08x ctx = %08x", 
+{
+	TRACE("got data count = %d offset = %d request = %08x origchannel = %08x ctx = %08x",
 	      count, off, request, &*channel, ctxt);
 	FK_ASSERT(debugNoHeaders == false);
-	
+
 	int action;
 	nsresult rv;
-	
+
 	sentence->GetAction(&action);
 
 	/*Whitelisted. Don't check it*/
-	if (action == sentence->DONT_AUDIT){		
+	if (action == sentence->DONT_AUDIT){
 		TRACE("whitelisted");
-		return origListener->OnDataAvailable(getRequestProxy(request), ctxt, input, 
+		return origListener->OnDataAvailable(tryGetRequestProxy(request), ctxt, input,
 						     off, count);
 	}
 	FK_ASSERT(action != sentence->BLOCK);
-	
+
 	if (expandBuf(&body, count, body_len, body_max) != NS_OK){
 		TRACE("Out of memory, can't expand buffer");
 		return NS_ERROR_OUT_OF_MEMORY;
 	}
 
-	PRUint32 res;	
+	PRUint32 res;
 	input->Read(body + body_len, count, &res);
-	FK_ASSERT(res == count);	
+	FK_ASSERT(res == count);
 	body_len += count;
-	
+
 	/*
 	  Not whole body fragment is filled.
 	 */
 	if (body_len < flush_point){
 		TRACE("fragment not filled");
-		return NS_OK;	
+		return NS_OK;
 	}
-	
+
 	/*Check current body fragment and pass it to the original listener*/
 	rv = do_OnDataAvailable(request, ctxt);
 	if (NS_FAILED(rv)){
 		TRACE("do_OnDataAvailable FAILED");
 		return rv;
-	}	
-	
+	}
+
 	drawFlushPoint();
 	drawNoFlushPoint();
 	return rv;
@@ -408,7 +412,7 @@ fkStreamListener::OnDataAvailable(nsIRequest *request, nsISupports *ctxt,
 NS_IMETHODIMP
 fkStreamListener::OnStartRequest(nsIRequest *request, nsISupports* acontext)
 {
-	TRACE("Request started %08x request = %08x origchannel = %08x ctx = %08x ", this, 
+	TRACE("Request started %08x request = %08x origchannel = %08x ctx = %08x ", this,
 	      request, channel, acontext);
 	nsresult rv = channel->VisitResponseHeaders(this);
 	if (NS_FAILED(rv)){
@@ -420,11 +424,11 @@ fkStreamListener::OnStartRequest(nsIRequest *request, nsISupports* acontext)
 	rv = headers_append("", 1);
 	if (NS_FAILED(rv)){
 		return rv;
-	}	
+	}
 
 	TRACE("headers:\n %s", headers);
-	
-	list<const Rule *> match = connRules->checkHeaders(headers, 
+
+	list<const Rule *> match = connRules->checkHeaders(headers,
 							   strlen(headers));
 	list<const Rule *>::iterator it;
 	for(it = match.begin(); it != match.end(); ++it){
@@ -432,9 +436,9 @@ fkStreamListener::OnStartRequest(nsIRequest *request, nsISupports* acontext)
 		if (!rule_raw){
 			return NS_ERROR_OUT_OF_MEMORY;
 		}
-		nsCOMPtr<fkIRule> 
+		nsCOMPtr<fkIRule>
 			rule(do_QueryInterface(rule_raw));
-		fkJudge->Judge(getURL(), rule, sentence);		
+		fkJudge->Judge(getURL(), rule, sentence);
 		int action;
 		sentence->GetAction(&action);
 		TRACE("action = %d", action);
@@ -449,7 +453,7 @@ fkStreamListener::OnStartRequest(nsIRequest *request, nsISupports* acontext)
 	}
 
 	TRACE("call");
-	rv = origListener->OnStartRequest(getRequestProxy(request), acontext);
+	rv = origListener->OnStartRequest(tryGetRequestProxy(request), acontext);
 	TRACE("returned");
 	return rv;
 }
@@ -461,24 +465,24 @@ NS_IMETHODIMP
 fkStreamListener::OnStopRequest(nsIRequest *request, nsISupports *acontext,
 				nsresult astatus)
 {
-	TRACE("Request stopped %08x status = %d request = %08x origchannel = %08x ctx = %08x", 
+	TRACE("Request stopped %08x status = %d request = %08x origchannel = %08x ctx = %08x",
 	      this, astatus, request, channel, acontext);
 	nsresult rv;
 	int action;
 	sentence->GetAction(&action);
-	
-	if (body_len && action != sentence->DONT_AUDIT && 
+
+	if (body_len && action != sentence->DONT_AUDIT &&
 	    action != sentence->BLOCK){
 		TRACE("passing remaining body fragment");
 		no_flush_point = 0;
-		rv = do_LastOnDataAvailable(request, acontext);	
+		rv = do_LastOnDataAvailable(request, acontext);
 		if (NS_FAILED(rv)){
-			TRACE("do_OnDataAvailable failed %d", rv);
+			TRACE("do_LastOnDataAvailable failed %d", rv);
 			astatus = rv;
 		}
 	}
-	
-	return origListener->OnStopRequest(getRequestProxy(request), acontext, astatus);
+
+	return origListener->OnStopRequest(tryGetRequestProxy(request), acontext, astatus);
 
 }
 
@@ -489,20 +493,19 @@ fkStreamListener::OnStopRequest(nsIRequest *request, nsISupports *acontext,
 
 /*Get single header and append it to headers buffer*/
 NS_IMETHODIMP
-fkStreamListener::VisitHeader(const nsACString &header, 
+fkStreamListener::VisitHeader(const nsACString &header,
 			      const nsACString &value)
 {
 
-	nsresult rv;	
+	nsresult rv;
 	PRBool null_ter;
 	const char *h, *v;
 	int hlen, vlen;
 	hlen = NS_CStringGetData(header, &h, &null_ter);
-	FK_ASSERT(null_ter == PR_TRUE);		
-	
+	FK_ASSERT(null_ter == PR_TRUE);
+
 	vlen = NS_CStringGetData(value, &v, &null_ter);
 	FK_ASSERT(null_ter == PR_TRUE);
-	
 
 	TRACE("h = %s v = %s", h, v);
 	rv = headers_append(h, hlen);
@@ -515,7 +518,7 @@ fkStreamListener::VisitHeader(const nsACString &header,
 	rv = headers_append(v, vlen);
 	if (NS_FAILED(rv))
 		return rv;
-	
+
 	rv = headers_append("\n", 1);
 	if (NS_FAILED(rv))
 		return rv;
