@@ -1,4 +1,4 @@
-/*$Id: Error.h,v 1.14 2006/11/04 21:09:26 jwrobel Exp $*/
+/*$Id: Error.h,v 1.16 2007/08/17 21:33:16 jwrobel Exp $*/
 /* ***** BEGIN LICENSE BLOCK *****
  *  This file is part of Firekeeper.
  *
@@ -40,8 +40,6 @@ void trace_func_windows(const char *fmt, ...);
 #ifdef DEBUG
 
 
-#define FK_ASSERT(x) if (!(x)) err_quit("ASSERTion %s in file %s line %d failed", #x, __FILE__, __LINE__);
-
 
 #ifndef MSVC
 
@@ -55,13 +53,16 @@ void trace_func_windows(const char *fmt, ...);
 
 
 #else /*MSVC*/
-/*How to append function name in Windows version of TRACE() macro?
-  Now it writes only debug message without function that produced it*/
-#define TRACE trace_func_windows
+
+extern const char *__funcname;
+
+#define TRACE __funcname = __FUNCTION__, trace_func_windows
 
 
 #endif /*MSVC*/
 
+#define FK_ASSERT(x) do{ if (!(x)){ TRACE("ASSERTion %s in file %s line %d failed. Press enter", #x, __FILE__, __LINE__); char c; scanf("%c", &c); exit(1);};} while(0)
+
 #else  /*DEBUG*/
 
 #ifndef MSVC
