@@ -1,4 +1,4 @@
-/*$Id: str_search.h,v 1.2 2006/10/06 19:31:40 jwrobel Exp $*/
+/*$Id: str_search.h,v 1.3 2007/03/23 11:49:03 jwrobel Exp $*/
 /*
  *  Copyright (C) 2006 Jan Wrobel
  *  Copyright (C) 2005 Sourcefire,Inc.
@@ -20,6 +20,9 @@ void SearchFree();
 void SearchAdd(unsigned int mpse_id, unsigned char *pat, unsigned int pat_len, 
 	       int nocase, int id);
 
+void SearchCompile(unsigned int mpse_id);
+void SearchCompileAll();
+
 typedef int (*MatchFunction)(void *, int, void *);
 
 int  SearchFindString(unsigned int mpse_id, unsigned char *str, 
@@ -39,6 +42,7 @@ typedef struct _search_api
 
 	int (*search_find)(unsigned int, unsigned char *, 
 			   unsigned int, MatchFunction, void *); 
+	void (*search_compile_all)();
 } SearchAPI;
 
 extern SearchAPI *searchAPI;
